@@ -42,6 +42,7 @@ const file_hint_t file_hint_exe= {
   .min_header_distance=0,
   .max_filesize=PHOTOREC_MAX_FILE_SIZE,
   .recover=1,
+  .enable_by_default=1,
   .register_header_check=&register_header_check_exe
 };
 
@@ -76,6 +77,7 @@ static int header_check_exe(const unsigned char *buffer, const unsigned int buff
     }
     if(le16(pe_hdr->Characteristics) & 0x2000)
     {
+      /* Dynamic Link Library */
       reset_file_recovery(file_recovery_new);
       file_recovery_new->extension="dll";
     }
