@@ -1,7 +1,7 @@
 /**
  * ntfs_dir.c - Part of the TestDisk project.
  *
- * Copyright (c) 2004-2007 Christophe Grenier
+ * Copyright (c) 2004-2008 Christophe Grenier
  *
  * Original version comes from the Linux-NTFS project.
  * Copyright (c) 2003 Lode Leroy
@@ -49,10 +49,15 @@
 #ifdef HAVE_MACHINE_ENDIAN_H
 #include <machine/endian.h>
 #endif
+#ifdef HAVE_ICONV_H
+#include <iconv.h>
+#endif
+#include <ctype.h>      /* isalpha */
 #include <stdarg.h>
 #include "types.h"
 
 #ifdef HAVE_LIBNTFS
+#include <ntfs/volume.h>
 #include <ntfs/attrib.h>
 #ifdef HAVE_NTFS_VERSION_H
 #include <ntfs/version.h>
@@ -64,7 +69,6 @@
 #include "ntfs.h"
 #include "dir.h"
 #include "ntfs_dir.h"
-#include "io_redir.h"
 #include "ntfs_utl.h"
 #include "ntfs_inc.h"
 #include "log.h"
@@ -75,6 +79,9 @@
 #define NTFS_DT_DIR               4
 #define NTFS_DT_REG             8
 #define NTFS_TIME_OFFSET ((s64)(369 * 365 + 89) * 24 * 3600 * 10000000)
+#ifndef FILE_first_user
+#define FILE_first_user 16
+#endif
 
 /*
  * This is the "ntfs_filldir" function type, used by ntfs_readdir() to let
@@ -93,7 +100,7 @@ extern int ntfs_readdir(ntfs_inode *dir_ni, s64 *pos,
 extern u64 ntfs_inode_lookup_by_name(ntfs_inode *dir_ni,
                 const ntfschar *uname, const int uname_len);
 static time_t ntfs2utc (s64 ntfstime);
-static int ntfs_list_entry(  struct ntfs_dir_struct *ls, const ntfschar *name, 
+static int ntfs_td_list_entry(  struct ntfs_dir_struct *ls, const ntfschar *name, 
 		const int name_len, const int name_type, const s64 pos,
 		const MFT_REF mref, const unsigned dt_type);
 static file_data_t *ntfs_dir(disk_t *disk_car, const partition_t *partition, dir_data_t *dir_data, const unsigned long int cluster);
@@ -137,11 +144,36 @@ static time_t ntfs2utc (s64 ntfstime)
   return (ntfstime - (NTFS_TIME_OFFSET)) / 10000000;
 }
 
+#ifdef HAVE_ICONV
+static int ntfs_ucstoutf8(iconv_t cd, const ntfschar *ins, int ins_len, char **outs, int outs_len)
+{
+    const char *inp;
+    char *outp;
+    size_t inb_left, outb_left;
+    if (cd == (iconv_t)(-1))
+      return -1;
+
+    outp = *outs;
+    inp = (const char *) ins;
+    inb_left = ins_len << 1;    // ntfschar is 16-bit
+    outb_left = outs_len - 1;   // reserve 1 byte for NUL
+
+    if (iconv(cd, (char**)&inp, &inb_left, &outp, &outb_left) == (size_t)(-1))
+    {
+      // Regardless of the value of errno
+      log_error("ntfs_ucstoutf8: iconv failed\n");
+      return -1;
+    }
+    *outp = '\0';
+    return 0;
+}
+#endif
+
 /**
- * ntfs_list_entry
+ * ntfs_td_list_entry
  * FIXME: Should we print errors as we go along? (AIA)
  */
-static int ntfs_list_entry(  struct ntfs_dir_struct *ls, const ntfschar *name, 
+static int ntfs_td_list_entry(  struct ntfs_dir_struct *ls, const ntfschar *name, 
 		const int name_len, const int name_type, const s64 pos,
 		const MFT_REF mref, const unsigned dt_type)
 {
@@ -150,18 +182,26 @@ static int ntfs_list_entry(  struct ntfs_dir_struct *ls, const ntfschar *name,
   filename = calloc (1, MAX_PATH);
   if (!filename)
   {
-    log_critical("ntfs_list_entry calloc failed\n");
+    log_critical("ntfs_td_list_entry calloc failed\n");
     return -1;
   }
 
+#ifdef HAVE_ICONV
+  if (ntfs_ucstoutf8(ls->cd, name, name_len, &filename, MAX_PATH) < 0 &&
+      ntfs_ucstombs (name, name_len, &filename, MAX_PATH) < 0) {
+    log_error("Cannot represent filename in current locale.\n");
+    goto free;
+  }
+#else
   if (ntfs_ucstombs (name, name_len, &filename, MAX_PATH) < 0) {
     log_error("Cannot represent filename in current locale.\n");
     goto free;
   }
+#endif
 
   result = 0;					/* These are successful */
-  if (filename[0] == '$')			/* system */
-    goto free;
+  if (MREF(mref) < FILE_first_user && filename[0] == '$')	/* Hide system file */
+      goto free;
   /* Keep FILE_NAME_WIN32 and FILE_NAME_POSIX */
   if ((name_type & FILE_NAME_WIN32_AND_DOS) == FILE_NAME_DOS)
     goto free;
@@ -201,8 +241,9 @@ static int ntfs_list_entry(  struct ntfs_dir_struct *ls, const ntfschar *name,
     }
 
     {
-      file_data_t *new_file=MALLOC(sizeof(*new_file));
+      file_data_t *new_file=(file_data_t *)MALLOC(sizeof(*new_file));
       memcpy(new_file->name,filename,(MAX_PATH<sizeof(new_file->name)?MAX_PATH:sizeof(new_file->name)));
+      new_file->status=0;
       new_file->prev=ls->current_file;
       new_file->next=NULL;
       new_file->filestat.st_dev=0;
@@ -227,9 +268,9 @@ static int ntfs_list_entry(  struct ntfs_dir_struct *ls, const ntfschar *name,
       new_file->next=NULL;
       /* log_debug("fat: new file %s de=%p size=%u\n",new_file->name,de,de->size); */
       if(ls->current_file!=NULL)
-	ls->current_file->next=new_file;
+        ls->current_file->next=new_file;
       else
-	ls->dir_list=new_file;
+        ls->dir_list=new_file;
       ls->current_file=new_file;
     }
 
@@ -246,32 +287,6 @@ free:
   return result;
 }
 
-/**
- * ntfs_mount - open ntfs volume
- * @name:	name of device/file to open
- * @rwflag:	optional mount flags
- *
- * This function mounts an ntfs volume. @name should contain the name of the
- * device/file to mount as the ntfs volume.
- *
- * @rwflags is an optional second parameter. The same flags are used as for
- * the mount system call (man 2 mount). Currently only the following flag
- * is implemented:
- *	MS_RDONLY	- mount volume read-only
- *
- * The function opens the device or file @name and verifies that it contains a
- * valid bootsector. Then, it allocates an ntfs_volume structure and initializes
- * some of the values inside the structure from the information stored in the
- * bootsector. It proceeds to load the necessary system files and completes
- * setting up the structure.
- *
- * Return the allocated volume structure on success and NULL on error with
- * errno set to the error code.
- *
- * Note, that a copy is made of @name, and hence it can be discarded as
- * soon as the function returns.
- */
-
 static file_data_t *ntfs_dir(disk_t *disk_car, const partition_t *partition, dir_data_t *dir_data, const unsigned long int cluster)
 {
   ntfs_inode *inode;
@@ -292,7 +307,7 @@ static file_data_t *ntfs_dir(disk_t *disk_car, const partition_t *partition, dir
    */
   pos = 0;
   if (inode->mrec->flags & MFT_RECORD_IS_DIRECTORY) {
-    if(ntfs_readdir(inode, &pos, ls, (ntfs_filldir_t)ntfs_list_entry)<0)
+    if(ntfs_readdir(inode, &pos, ls, (ntfs_filldir_t)ntfs_td_list_entry)<0)
     {
       log_error("ntfs_readdir failed\n");
     }
@@ -323,7 +338,7 @@ static int ntfs_copy(disk_t *disk_car, const partition_t *partition, dir_data_t
     s64 bytes_read, written;
     s64 offset;
     u32 block_size;
-    buffer = MALLOC(bufsize);
+    buffer = (char *)MALLOC(bufsize);
     if (!buffer)
       return -2;
     attr = ntfs_attr_open(inode, AT_DATA, NULL, 0);
@@ -339,9 +354,7 @@ static int ntfs_copy(disk_t *disk_car, const partition_t *partition, dir_data_t
       block_size = index_get_size(inode);
     else
       block_size = 0;
-    new_file=MALLOC(strlen(dir_data->local_dir)+strlen(dir_data->current_directory)+1);
-    strcpy(new_file,dir_data->local_dir);
-    strcat(new_file,dir_data->current_directory);
+    new_file=gen_local_filename(dir_data->local_dir, dir_data->current_directory);
     f_out=create_file(new_file);
     if(!f_out)
     {
@@ -396,20 +409,20 @@ static void dir_partition_ntfs_close(dir_data_t *dir_data)
   partition=ls->my_data->partition;
   ntfs_umount(ls->vol, FALSE);
   free(ls->my_data);
-  if(partition->boot_sector!=0)
-    io_redir_del_redir(disk_car,partition->part_offset);
+#ifdef HAVE_ICONV
+  if (ls->cd != (iconv_t)(-1))
+    iconv_close(ls->cd);
+#endif
   free(ls);
 }
 #endif
 
-int dir_partition_ntfs_init(disk_t *disk_car, const partition_t *partition, dir_data_t *dir_data, const int debug)
+int dir_partition_ntfs_init(disk_t *disk_car, const partition_t *partition, dir_data_t *dir_data, const int verbose)
 {
 #ifdef HAVE_LIBNTFS
   struct ntfs_device *dev;
   my_data_t *my_data=NULL;
   ntfs_volume *vol=NULL;
-  if(partition->boot_sector!=0)
-    io_redir_add_redir(disk_car,partition->part_offset,DEFAULT_SECTOR_SIZE,partition->part_offset+(uint64_t)partition->boot_sector*disk_car->sector_size,NULL);
 #ifdef NTFS_LOG_LEVEL_VERBOSE
   ntfs_log_set_levels(NTFS_LOG_LEVEL_VERBOSE);
   ntfs_log_set_handler(ntfs_log_handler_stderr);
@@ -418,16 +431,24 @@ int dir_partition_ntfs_init(disk_t *disk_car, const partition_t *partition, dir_
   dev = ntfs_device_alloc("/", 0, &ntfs_device_testdisk_io_ops, NULL);
   if (dev)
   {
-    my_data=MALLOC(sizeof(*my_data));
+    my_data=(my_data_t *)MALLOC(sizeof(*my_data));
     my_data->partition=partition;
     my_data->disk_car=disk_car;
     my_data->offset=0;
     dev->d_private=my_data;
     /* Call ntfs_device_mount() to do the actual mount. */
+#ifdef MS_RDONLY
     vol = ntfs_device_mount(dev, MS_RDONLY);
+#else
+    vol = ntfs_device_mount(dev, NTFS_MNT_RDONLY);
+#endif
 #ifdef HAVE_NTFS_VOLUME_STARTUP
     if(!vol) {
+#ifdef MS_RDONLY
       vol = ntfs_volume_startup(dev, MS_RDONLY);
+#else
+      vol = ntfs_volume_startup(dev, NTFS_MNT_RDONLY);
+#endif
       if(vol)
       {
 	log_warning("NTFS filesystem need to be repaired.\n");
@@ -436,8 +457,6 @@ int dir_partition_ntfs_init(disk_t *disk_car, const partition_t *partition, dir_
 #endif
   }
   if (!vol) {
-    if(partition->boot_sector!=0)
-      io_redir_del_redir(disk_car,partition->part_offset);
     free(my_data);
     ntfs_device_free(dev);
     return -1;
@@ -451,9 +470,17 @@ int dir_partition_ntfs_init(disk_t *disk_car, const partition_t *partition, dir_
     ls->current_file=NULL;
     ls->vol=vol;
     ls->my_data=my_data;
+#ifdef HAVE_ICONV
+    if ((ls->cd = iconv_open("UTF-8", "UTF-16LE")) == (iconv_t)(-1))
+    {
+      log_error("ntfs_ucstoutf8: iconv_open failed\n");
+    }
+#endif
     strncpy(dir_data->current_directory,"/",sizeof(dir_data->current_directory));
     dir_data->current_inode=FILE_root;
-    dir_data->debug=debug;
+    dir_data->param=0;
+    dir_data->verbose=verbose;
+    dir_data->capabilities=0;
     dir_data->get_dir=ntfs_dir;
     dir_data->copy_file=ntfs_copy;
     dir_data->close=&dir_partition_ntfs_close;
