@@ -2,7 +2,7 @@
 
     File: partnone.c
 
-    Copyright (C) 1998-2007 Christophe GRENIER <grenier@cgsecurity.org>
+    Copyright (C) 1998-2008 Christophe GRENIER <grenier@cgsecurity.org>
 
     This software is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -68,7 +68,6 @@ static int check_part_none(disk_t *disk_car, const int verbose,partition_t *part
 static int get_geometry_from_nonembr(const unsigned char *buffer, const int verbose, CHS_t *geometry);
 static list_part_t *read_part_none(disk_t *disk_car, const int verbose, const int saveheader);
 static list_part_t *init_part_order_none(const disk_t *disk_car, list_part_t *list_part);
-static list_part_t *add_partition_none(disk_t *disk_car,list_part_t *list_part, const int verbose, char **current_cmd);
 static void set_next_status_none(const disk_t *disk_car, partition_t *partition);
 static int test_structure_none(list_part_t *list_part);
 static int is_part_known_none(const partition_t *partition);
@@ -80,6 +79,7 @@ static const struct systypes none_sys_types[] = {
   {UP_CRAMFS,	"CramFS"},
   {UP_EXT2,	"ext2"},
   {UP_EXT3,	"ext3"},
+  {UP_EXT4,	"ext4"},
 /*  {UP_EXTENDED,	"Extended"}, */
   {UP_FAT12,	"FAT12"},
   {UP_FAT16,	"FAT16"},
@@ -128,7 +128,7 @@ arch_fnct_t arch_none=
   .get_geometry_from_mbr=get_geometry_from_nonembr,
   .check_part=check_part_none,
   .write_MBR_code=NULL,
-  .add_partition=add_partition_none,
+  .add_partition=NULL,
   .set_prev_status=set_next_status_none,
   .set_next_status=set_next_status_none,
   .test_structure=test_structure_none,
@@ -171,7 +171,7 @@ list_part_t *read_part_none(disk_t *disk_car, const int verbose, const int saveh
   partition_t *partition;
   int res=0;
   partition=partition_new(&arch_none);
-  buffer_disk=MALLOC(16*DEFAULT_SECTOR_SIZE);
+  buffer_disk=(unsigned char *)MALLOC(16*DEFAULT_SECTOR_SIZE);
   partition->part_size=disk_car->disk_size;
   if(recover_MD_from_partition(disk_car, partition, verbose)==0)
     res=1;
@@ -197,7 +197,7 @@ list_part_t *read_part_none(disk_t *disk_car, const int verbose, const int saveh
   partition->part_size=disk_car->disk_size;
   partition->order=NO_ORDER;
   partition->status=STATUS_PRIM;
-  aff_buffer(BUFFER_RESET,"Q");
+  screen_buffer_reset();
   disk_car->arch->check_part(disk_car,verbose,partition,saveheader);
   aff_part_buffer(AFF_PART_ORDER|AFF_PART_STATUS,disk_car,partition);
   list_part=insert_new_partition(NULL, partition, 0, &insert_error);
@@ -211,10 +211,6 @@ static list_part_t *init_part_order_none(const disk_t *disk_car, list_part_t *li
   return list_part;
 }
 
-static list_part_t *add_partition_none(disk_t *disk_car,list_part_t *list_part, const int verbose, char **current_cmd)
-{
-  return list_part;
-}
 
 static void set_next_status_none(const disk_t *disk_car, partition_t *partition)
 {
@@ -227,7 +223,7 @@ static int test_structure_none(list_part_t *list_part)
 
 int set_part_type_none(partition_t *partition, unsigned int part_type)
 {
-  partition->upart_type=part_type;
+  partition->upart_type=(upart_type_t)part_type;
   return 0;
 }
 
@@ -258,6 +254,7 @@ static int check_part_none(disk_t *disk_car,const int verbose,partition_t *parti
       break;
     case UP_EXT2:
     case UP_EXT3:
+    case UP_EXT4:
       ret=check_EXT2(disk_car,partition,verbose);
       break;
     case UP_EXTENDED:
@@ -302,7 +299,7 @@ static int check_part_none(disk_t *disk_car,const int verbose,partition_t *parti
     case UP_NTFS:
       ret=check_NTFS(disk_car,partition,verbose,0);
       if(ret!=0)
-      { aff_buffer(BUFFER_ADD,"Invalid NTFS boot\n"); }
+      { screen_buffer_add("Invalid NTFS boot\n"); }
       break;
     case UP_OPENBSD:
       ret=check_BSD(disk_car,partition,verbose,OPENBSD_MAXPARTITIONS);
@@ -314,7 +311,7 @@ static int check_part_none(disk_t *disk_car,const int verbose,partition_t *parti
     case UP_MD1:
       ret=check_MD(disk_car,partition,verbose);
       if(ret!=0)
-      { aff_buffer(BUFFER_ADD,"Invalid RAID superblock\n"); }
+      { screen_buffer_add("Invalid RAID superblock\n"); }
       break;
     case UP_RFS:
     case UP_RFS2:
