@@ -2,7 +2,7 @@
 
     File: testdisk.c
 
-    Copyright (C) 1998-2007 Christophe GRENIER <grenier@cgsecurity.org>
+    Copyright (C) 1998-2008 Christophe GRENIER <grenier@cgsecurity.org>
   
     This software is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -62,6 +62,7 @@
 #include "hdaccess.h"
 #include "sudo.h"
 #include "partauto.h"
+#include "misc.h"
 
 extern const arch_fnct_t arch_i386;
 extern const arch_fnct_t arch_mac;
@@ -94,7 +95,7 @@ int main( int argc, char **argv )
 {
   int i;
   int use_sudo=0;
-  int help=0, verbose=0, dump_ind=0;
+  int help=0, version=0, verbose=0, dump_ind=0;
   int create_log=TD_LOG_NONE;
   int do_list=0;
   int unit=UNIT_DEFAULT;
@@ -156,6 +157,9 @@ int main( int argc, char **argv )
     else if((strcmp(argv[i],"/help")==0) || (strcmp(argv[i],"-help")==0) || (strcmp(argv[i],"--help")==0) ||
       (strcmp(argv[i],"/h")==0) || (strcmp(argv[i],"-h")==0))
       help=1;
+    else if((strcmp(argv[i],"/version")==0) || (strcmp(argv[i],"-version")==0) || (strcmp(argv[i],"--version")==0) ||
+      (strcmp(argv[i],"/v")==0) || (strcmp(argv[i],"-v")==0))
+      version=1;
     else if(strcmp(argv[i],"/list")==0 || strcmp(argv[i],"-list")==0 || strcmp(argv[i],"-l")==0)
     {
       do_list=1;
@@ -202,11 +206,22 @@ int main( int argc, char **argv )
 	list_disk=insert_new_disk(list_disk,disk_car);
     }
   }
+  if(version!=0)
+  {
+    printf("\n");
+    printf("Version: %s\n", VERSION);
+    printf("Compiler: %s\n", get_compiler());
+    printf("ext2fs lib: %s, ntfs lib: %s, reiserfs lib: %s, ewf lib: %s\n",
+	td_ext2fs_version(), td_ntfs_version(), td_reiserfs_version(), td_ewf_version());
+    printf("OS: %s\n" , get_os());
+    return 0;
+  }
   if(help!=0)
   {
     printf("\n" \
-	"Usage: testdisk [/log] [/debug] [file or device]\n"\
-	"       testdisk /list  [/log]   [file or device]\n" \
+	"Usage: testdisk [/log] [/debug] [file.dd|file.e01|device]\n"\
+	"       testdisk /list  [/log]   [file.dd|file.e01|device]\n" \
+	"       testdisk /version\n" \
 	"\n" \
 	"/log          : create a testdisk.log file\n" \
 	"/debug        : add debug information\n" \
@@ -226,7 +241,7 @@ int main( int argc, char **argv )
 	"If you have problems with TestDisk or bug reports, please contact me.\n");
     return 0;
   }
-  aff_buffer(BUFFER_RESET,"Q");
+  screen_buffer_reset();
   if(do_list!=0)
   {
     printf("Please wait...\n");
@@ -286,9 +301,11 @@ int main( int argc, char **argv )
     create_log=ask_testdisk_log_creation();
   }
   create_log=log_open("testdisk.log", create_log, 1, "TestDisk", argc, argv);
-  log_info("TestDisk %s, Data Recovery Utility, %s\nChristophe GRENIER <grenier@cgsecurity.org>\nhttp://www.cgsecurity.org\n",VERSION,TESTDISKDATE);
-  log_info(TESTDISK_OS);
-  log_info(" (ext2fs lib: %s, ntfs lib: %s, reiserfs lib: %s, ewf lib: %s)\n",td_ext2fs_version(),td_ntfs_version(),td_reiserfs_version(), td_ewf_version());
+  log_info("TestDisk %s, Data Recovery Utility, %s\nChristophe GRENIER <grenier@cgsecurity.org>\nhttp://www.cgsecurity.org\n", VERSION, TESTDISKDATE);
+  log_info("OS: %s\n" , get_os());
+  log_info("Compiler: %s\n", get_compiler());
+  log_info("ext2fs lib: %s, ntfs lib: %s, reiserfs lib: %s, ewf lib: %s\n",
+      td_ext2fs_version(), td_ntfs_version(), td_reiserfs_version(), td_ewf_version());
 #if defined(__CYGWIN__) || defined(__MINGW32__) || defined(DJGPP)
 #else
 #ifdef HAVE_GETEUID
